import time
import jwt
from kuaishou.vod.core.exception.kuaishou_vod_sdk_exception import KuaishouVodSDKException


class PlayClient(object):
    def generate_play_token(self, play_token_req, credential):
        start_time = int(round(time.time() * 1000))
        domain_key = play_token_req.domain_key
        duration = play_token_req.duration
        result = credential.generate_sts_token(duration)
        if result is None:
            raise KuaishouVodSDKException('Auth failed', 'Invalid Authorization, ak=' + credential.access_key_id)
        sts_token = result["sessionToken"]
        expire_time = result["expiration"]
        tmp_access_key = result["accessKeyId"]
        tmp_secret_key = result["accessKeySecret"]
        sts_map = {
            "TempAccessKey": tmp_access_key,
            "TempSecretKey": tmp_secret_key,
            "SecurityToken": sts_token,
            "CreateTime": int(time.time()),
            "ExpireTime": int(expire_time/1000),
            "StsTokenVer": 2
        }
        url_auth_map = {}
        url_auth_info = play_token_req.url_auth_info
        if url_auth_info.expire_time is not None and url_auth_info.expire_time > 0:
            url_auth_map["ExpireTime"] = url_auth_info.expire_time

        payload = {
            "StsToken": sts_map,
            "Domain": play_token_req.domain,
            "UrlAuthInfo": url_auth_map
        }
        if play_token_req.media_id is not None:
            payload["MediaId"] = play_token_req.media_id

        play_token = jwt.encode(
            payload=payload,
            key=domain_key,
            algorithm='HS256'
        )
        end_time = int(round(time.time() * 1000))
        return play_token
