# -*- coding: utf-8 -*-
import hashlib
import hmac
import time
from datetime import datetime


class SignHelper(object):

    @staticmethod
    def sign(signature):
        access_key_id = signature.access_key_id
        access_key_secret = signature.access_key_secret
        service = signature.service
        host = signature.host
        content_type = signature.content_type
        region = signature.region
        action = signature.action
        version = signature.version
        algorithm = signature.algorithm
        timestamp = int(time.time())
        date = datetime.utcfromtimestamp(timestamp).strftime("%Y-%m-%d")

        # ************* 步骤 1：拼接规范请求串 *************
        request_method = signature.request_method
        canonicalUri = "/"
        canonicalQueryString = signature.canonical_query_string
        canonical_headers = signature.canonical_headers
        signed_headers = signature.signed_headers
        payload = signature.payload
        hashed_request_payload = hashlib.sha256(payload.encode("utf-8")).hexdigest()
        canonical_request = \
            request_method + "\n" \
            + canonicalUri + "\n" \
            + canonicalQueryString + "\n" \
            + canonical_headers + "\n" \
            + signed_headers + "\n" \
            + hashed_request_payload

        # ************* 步骤 2：拼接待签名字符串 *************
        credential_scope = date + "/" + service + "/" + "sl_request"
        hashed_canonical_request = hashlib.sha256(canonical_request.encode('utf-8')).hexdigest()
        string_to_sign = (algorithm + "\n" +
                          str(timestamp) + "\n" +
                          credential_scope + "\n" +
                          hashed_canonical_request)

        # ************* 步骤 3：计算签名 *************
        # 计算签名摘要函数
        def sign(key, msg):
            return hmac.new(key, msg.encode("utf-8"), hashlib.sha256).digest()

        secret_date = sign(("SL" + access_key_secret).encode("utf-8"), date)
        secret_service = sign(secret_date, service)
        secret_signing = sign(secret_service, "sl_request")
        signature = hmac.new(secret_signing, string_to_sign.encode("utf-8"), hashlib.sha256).hexdigest()

        # ************* 步骤 4：拼接 Authorization *************
        authorization = (algorithm + " " +
                         "Credential=" + access_key_id + "/" + credential_scope + ", " +
                         "SignedHeaders=" + signed_headers + ", " +
                         "Signature=" + signature + "sl_request")

        return {
            "Authorization": authorization,
            "Content-Type": content_type,
            "Host": host,
            "X-SL-Action": action,
            "X-SL-Timestamp": str(timestamp),
            "X-SL-Version": version,
            "X-SL-Program_Language": "Python",
            "SignatureVersion": "1",
            "AccessKey": access_key_id
        }
