import json

from kuaishou.vod.core.auth.sign_helper import SignHelper
from kuaishou.vod.core.auth.signature import Signature
from kuaishou.vod.core.http.content_type import *
from kuaishou.vod.core.http.http_profile import HttpProfile
from kuaishou.vod.core.http.request import ConnectionConfig
from kuaishou.vod.core.http.request import RequestInternal

try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode


class Client(object):

    def __init__(self, credential=None, profile=None):
        self.credential = credential
        self.profile = HttpProfile() if profile is None else profile
        self.request = ConnectionConfig(self.profile.reqTimeout)

    def _build_req(self, req, content_type, request):
        if req.method == 'GET':
            content_type = APPLICATION_JSON
        elif req.method == 'POST':
            content_type = content_type
        else:
            raise Exception("Request method only supports GET and POST")
        if content_type == APPLICATION_FORM:
            req.data = urlencode(request.serialize())
        elif content_type == APPLICATION_JSON:
            req.data = request.get_body_params()
        req.headers["Content-Type"] = content_type
        req.headers["AccessKey"] = self.credential.get_app_id()
        req.content_type = content_type
        # for prt
        # req.headers["trace-context"] = "{\"laneId\":\"PRT.streamlake\"}"

    def _get_service_domain(self, action):
        return 'https://' + self.profile.endpoint + "/?Action=" + action

    def _get_endpoint(self):
        return self.profile.endpoint

    def call(self, action, method, request, content_type=APPLICATION_FORM, service='vod'):
        req_inter = RequestInternal(self._get_endpoint(), method, self._get_service_domain(action))
        self._build_req(req_inter, content_type, request)
        self.sign(req_inter, action, method, req_inter.data, content_type, service=service)
        resp_inter = self.request.send_request(req_inter)
        return resp_inter.data

    def sign(self, req, action, method, payload, content_type, service):
        signature = Signature()
        signature.access_key_id = self.credential.get_app_id()
        signature.access_key_secret = self.credential.get_app_secret()
        signature.algorithm = 'SL-HMAC-SHA256'
        signature.action = action
        signature.host = self.profile.endpoint
        signature.content_type = content_type
        signature.region = 'beijing'
        signature.version = '2022-06-23'
        signature.service = service
        signature.canonical_headers = "content-type:" + content_type + "\nhost:" + self.profile.endpoint
        signature.canonical_query_string = "Action=" + action
        signature.request_method = method
        signature.signed_headers = "content-type;host"
        if content_type == APPLICATION_FORM:
            signature.payload = payload
        elif content_type == APPLICATION_JSON:
            signature.payload = json.dumps(payload)
        headers = SignHelper.sign(signature)
        for key in headers:
            req.headers[key] = headers[key]
